// Copyright (C) 2000 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// As a special exception to the GNU General Public License, permission is
// granted for additional uses of the text contained in its release
// of Bayonne as noted here.
//
// This exception is that permission is hereby granted to link Bayonne 
// with  the Pika MonteCarlo libraries to produce a executable image
// without requiring MonteCarlo itself to be supplied in source form so
// long as each source file so linked contains this exclusion.
//
// This exception does not however invalidate any other reasons why
// the resulting executable file might be covered by the GNU General
// public license or invalidate the licensing requirements of any
// other component or library.
//
// This exception applies only to the code released by OST under the
// name Bayonne.  If you copy code from other releases into a copy of
// Bayonne, as the General Public License permits, the exception does not
// apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
//
// If you write modifications of your own to Bayonne, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice, at which
// point the terms of your modification would be covered under the GPL
// as explicitly stated in "COPYING".

#include "driver.h"

PikaConf::PikaConf(TResourceHandle hDsp, unsigned cid)
{
	unsigned mem = 0;
	
	members = used = 0;
	member = NULL;
	id = cid;

	PK_VC_ChangeParameters(hDsp, PK_VC_GAIN_ROBUSTNESS, PK_VC_LONG_DISTANCE_ROBUSTNESS);
	hConf = PK_VC_CreateGroup(hDsp);
	if(!hConf)
		return;

	members = pikaivr.getConfMembers();
	member = new TConfConnection[members];
	memset(&member, sizeof(TConfConnection) * members, 0);

	while(mem < members)
	{
		member[mem].hConfPort = PK_DSP_SeizePort(hDsp, PK_VOICE_CONFERENCE);
		if(!member[mem].hConfPort)
			break;
		++mem;
	}
	members = mem;
	if(members)
		slog(SLOG_DEBUG) << "conf(" << id << "): created " << members << " member resource" << endl;
}

PikaConf::~PikaConf()
{
	unsigned mem = 0;

	if(!member)
		return;

	if(members)
		slog(SLOG_DEBUG) << "conf(" << id << "): removed " << members << " member resource" << endl;	

	while(mem < members)
		PK_DSP_ReleasePort(member[mem++].hConfPort);

//	PK_VC_DeleteGroup(hConf);

	delete[] member;
}		

bool PikaConf::Connect(PikaTrunk *trunk, bool duplex)
{
	unsigned mem = 0;

	EnterMutex();
	while(mem < members)
	{
		if(member[mem].hConnectedTo)
		{
			++mem;
			continue;
		}
		member[mem].hConnectedTo = trunk->hTrk;
		if(!trunk->hDsp)
			trunk->getVPResource();
		PK_MVIP_Disconnect(trunk->hDsp, trunk->hTrk);
		if(duplex)
			PK_MVIP_FullDuplexConnect(member[mem].hConfPort, member[mem].hConnectedTo);
		else
		{
			PK_MVIP_HalfDuplexConnect(member[mem].hConfPort, trunk->hTrk);
			PK_MVIP_HalfDuplexConnect(trunk->hTrk, trunk->hDsp);
		}
		PK_VC_AddMember(hConf, member[mem].hConfPort, 0, 0, PK_FALSE);
		++used;
		trunk->conf = this;
		LeaveMutex();
		return true;
	}
	LeaveMutex();
	return false;
}
	

void PikaConf::Disconnect(PikaTrunk *trunk)
{
	unsigned mem = 0;

	EnterMutex();
	while(mem < members)
	{
		if(member[mem].hConnectedTo != trunk->hTrk)
		{
			++mem;
			continue;
		}
		PK_VC_RemoveMember(hConf, member[mem].hConfPort);
		PK_MVIP_Disconnect(member[mem].hConnectedTo, member[mem].hConfPort);
		member[mem].hConnectedTo = 0;
		--used;

		trunk->conf = NULL;
		if(trunk->hDsp)
			PK_MVIP_FullDuplexConnect(trunk->hDsp, trunk->hTrk);
		else
			trunk->flags.dsp = DSP_MODE_INACTIVE;
		break;
	}
	LeaveMutex();
}

